@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -111,6 +111,15 @@ public abstract class AbstractCIFSAccess {
         this.auth = auth;
     }
 
+    /**
+     * Gets the root URL.
+     *
+     * @return The root URL
+     */
+    public String getRootUrl() {
+        return rootUrl;
+    }
+
     /**
      * Gets the associated SMB file.
      *
