@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,9 +53,10 @@ import gnu.trove.map.TIntObjectMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
 import jcifs.smb.SmbException;
 import com.openexchange.exception.Category;
+import com.openexchange.exception.DisplayableOXExceptionCode;
 import com.openexchange.exception.OXException;
-import com.openexchange.exception.OXExceptionCode;
 import com.openexchange.exception.OXExceptionFactory;
+import com.openexchange.exception.OXExceptionStrings;
 
 /**
  * {@link CIFSExceptionCodes} - Enumeration of all {@link CIFSException}s.
@@ -63,303 +64,337 @@ import com.openexchange.exception.OXExceptionFactory;
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since Open-Xchange v6.18.2
  */
-public enum CIFSExceptionCodes implements OXExceptionCode {
+public enum CIFSExceptionCodes implements DisplayableOXExceptionCode {
 
     /**
      * An error occurred: %1$s
      */
-    UNEXPECTED_ERROR(CIFSExceptionMessages.UNEXPECTED_ERROR_MSG, CATEGORY_ERROR),
+    UNEXPECTED_ERROR("An error occurred: %1$s", CATEGORY_ERROR, null),
     /**
      * A CIFS/SMB error occurred: %1$s
      */
-    SMB_ERROR(CIFSExceptionMessages.SMB_ERROR_MSG, CATEGORY_USER_INPUT),
+    SMB_ERROR("A CIFS/SMB error occurred: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.SMB_ERROR_MSG),
     /**
      * A CIFS/SMB error occurred: %1$s (status=%2$s)
      */
-    SMB_ERROR_WITH_STATUS(CIFSExceptionMessages.SMB_ERROR_WITH_STATUS, CATEGORY_USER_INPUT),
+    SMB_ERROR_WITH_STATUS("A CIFS/SMB error occurred: %1$s (status=%2$s)", CATEGORY_USER_INPUT, CIFSExceptionMessages.SMB_ERROR_MSG),
     /**
      * Invalid CIFS/SMB URL: %1$s
      */
-    INVALID_SMB_URL(CIFSExceptionMessages.INVALID_SMB_URL_MSG, CATEGORY_USER_INPUT),
+    INVALID_SMB_URL("Invalid CIFS/SMB URL: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.INVALID_SMB_URL_MSG),
     /**
      * CIFS/SMB URL does not denote a directory: %1$s
      */
-    NOT_A_FOLDER(CIFSExceptionMessages.NOT_A_FOLDER_MSG, CATEGORY_USER_INPUT),
+    NOT_A_FOLDER("CIFS/SMB URL does not denote a directory: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.NOT_A_FOLDER_MSG),
     /**
      * The CIFS/SMB resource does not exist: %1$s
      */
-    NOT_FOUND(CIFSExceptionMessages.NOT_FOUND_MSG, CATEGORY_USER_INPUT),
+    NOT_FOUND("The CIFS/SMB resource does not exist: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.NOT_FOUND_MSG),
     /**
      * Update denied for CIFS/SMB resource: %1$s
      */
-    UPDATE_DENIED(CIFSExceptionMessages.UPDATE_DENIED_MSG, CATEGORY_USER_INPUT),
+    UPDATE_DENIED("Update denied for CIFS/SMB resource: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.UPDATE_DENIED_MSG),
     /**
      * Delete denied for CIFS/SMB resource: %1$s
      */
-    DELETE_DENIED(CIFSExceptionMessages.DELETE_DENIED_MSG, CATEGORY_USER_INPUT),
+    DELETE_DENIED("Delete denied for CIFS/SMB resource: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.DELETE_DENIED_MSG),
     /**
      * CIFS/SMB URL does not denote a file: %1$s
      */
-    NOT_A_FILE(CIFSExceptionMessages.NOT_A_FILE_MSG, CATEGORY_USER_INPUT),
+    NOT_A_FILE("CIFS/SMB URL does not denote a file: %1$s", CATEGORY_USER_INPUT, CIFSExceptionMessages.NOT_A_FILE_MSG),
     /**
      * Missing file name.
      */
-    MISSING_FILE_NAME(CIFSExceptionMessages.MISSING_FILE_NAME_MSG, CATEGORY_USER_INPUT),
+    MISSING_FILE_NAME("Missing file name.", CATEGORY_USER_INPUT, CIFSExceptionMessages.MISSING_FILE_NAME_MSG),
     /**
      * Versioning not supported by CIFS/SMB file storage.
      */
-    VERSIONING_NOT_SUPPORTED(CIFSExceptionMessages.VERSIONING_NOT_SUPPORTED_MSG, CATEGORY_USER_INPUT),
+    VERSIONING_NOT_SUPPORTED("Versioning not supported by CIFS/SMB file storage.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.VERSIONING_NOT_SUPPORTED_MSG),
     /**
      * The operation completed successfully.
      */
-    NT_STATUS_OK(CIFSExceptionMessages.NT_STATUS_OK, CATEGORY_USER_INPUT),
+    NT_STATUS_OK("The operation completed successfully.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_OK),
     /**
      * A device attached to the system is not functioning.
      */
-    NT_STATUS_UNSUCCESSFUL(CIFSExceptionMessages.NT_STATUS_UNSUCCESSFUL, CATEGORY_USER_INPUT),
+    NT_STATUS_UNSUCCESSFUL("A device attached to the system is not functioning.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_UNSUCCESSFUL),
     /**
      * Incorrect function.
      */
-    NT_STATUS_NOT_IMPLEMENTED(CIFSExceptionMessages.NT_STATUS_NOT_IMPLEMENTED, CATEGORY_USER_INPUT),
+    NT_STATUS_NOT_IMPLEMENTED("Incorrect function.", CATEGORY_ERROR, CIFSExceptionMessages.NT_STATUS_NOT_IMPLEMENTED),
     /**
      * The parameter is incorrect.
      */
-    NT_STATUS_INVALID_INFO_CLASS(CIFSExceptionMessages.NT_STATUS_INVALID_INFO_CLASS, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_INFO_CLASS("The parameter is incorrect.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_INVALID_INFO_CLASS),
     /**
      * Invalid access to memory location.
      */
-    NT_STATUS_ACCESS_VIOLATION(CIFSExceptionMessages.NT_STATUS_ACCESS_VIOLATION, CATEGORY_USER_INPUT),
+    NT_STATUS_ACCESS_VIOLATION("Invalid access to memory location.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_ACCESS_VIOLATION),
     /**
      * The handle is invalid.
      */
-    NT_STATUS_INVALID_HANDLE(CIFSExceptionMessages.NT_STATUS_INVALID_HANDLE, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_HANDLE("The handle is invalid.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_INVALID_HANDLE),
     /**
      * The parameter is incorrect.
      */
-    NT_STATUS_INVALID_PARAMETER(CIFSExceptionMessages.NT_STATUS_INVALID_PARAMETER, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_PARAMETER("The parameter is incorrect.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_INVALID_PARAMETER),
     /**
      * The system cannot find the file specified.
      */
-    NT_STATUS_NO_SUCH_DEVICE(CIFSExceptionMessages.NT_STATUS_NO_SUCH_DEVICE, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_SUCH_DEVICE("The system cannot find the file specified.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_NO_SUCH_DEVICE),
     /**
      * The system cannot find the file specified.
      */
-    NT_STATUS_NO_SUCH_FILE(CIFSExceptionMessages.NT_STATUS_NO_SUCH_FILE, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_SUCH_FILE("The system cannot find the file specified.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NO_SUCH_FILE),
     /**
      * More data is available.
      */
-    NT_STATUS_MORE_PROCESSING_REQUIRED(CIFSExceptionMessages.NT_STATUS_MORE_PROCESSING_REQUIRED, CATEGORY_USER_INPUT),
+    NT_STATUS_MORE_PROCESSING_REQUIRED("More data is available.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_MORE_PROCESSING_REQUIRED),
     /**
      * Access is denied.
      */
-    NT_STATUS_ACCESS_DENIED(CIFSExceptionMessages.NT_STATUS_ACCESS_DENIED, CATEGORY_USER_INPUT),
+    NT_STATUS_ACCESS_DENIED("Access is denied.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_ACCESS_DENIED),
     /**
      * The data area passed to a system call is too small.
      */
-    NT_STATUS_BUFFER_TOO_SMALL(CIFSExceptionMessages.NT_STATUS_BUFFER_TOO_SMALL, CATEGORY_USER_INPUT),
+    NT_STATUS_BUFFER_TOO_SMALL("The data area passed to a system call is too small.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_BUFFER_TOO_SMALL),
     /**
      * The filename, directory name, or volume label syntax is incorrect.
      */
-    NT_STATUS_OBJECT_NAME_INVALID(CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_INVALID, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_NAME_INVALID("The filename, directory name, or volume label syntax is incorrect.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_INVALID),
     /**
      * The system cannot find the file specified.
      */
-    NT_STATUS_OBJECT_NAME_NOT_FOUND(CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_NOT_FOUND, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_NAME_NOT_FOUND("The system cannot find the file specified.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_NOT_FOUND),
     /**
      * Cannot create a file when that file already exists.
      */
-    NT_STATUS_OBJECT_NAME_COLLISION(CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_COLLISION, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_NAME_COLLISION("Cannot create a file when that file already exists.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_NAME_COLLISION),
     /**
      * The handle is invalid.
      */
-    NT_STATUS_PORT_DISCONNECTED(CIFSExceptionMessages.NT_STATUS_PORT_DISCONNECTED, CATEGORY_USER_INPUT),
+    NT_STATUS_PORT_DISCONNECTED("The handle is invalid.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_PORT_DISCONNECTED),
     /**
      * The specified path is invalid.
      */
-    NT_STATUS_OBJECT_PATH_INVALID(CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_INVALID, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_PATH_INVALID("The specified path is invalid.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_INVALID),
     /**
      * The system cannot find the path specified.
      */
-    NT_STATUS_OBJECT_PATH_NOT_FOUND(CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_NOT_FOUND, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_PATH_NOT_FOUND("The system cannot find the path specified.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_NOT_FOUND),
     /**
      * The specified path is invalid.
      */
-    NT_STATUS_OBJECT_PATH_SYNTAX_BAD(CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_SYNTAX_BAD, CATEGORY_USER_INPUT),
+    NT_STATUS_OBJECT_PATH_SYNTAX_BAD("The specified path is invalid.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_OBJECT_PATH_SYNTAX_BAD),
     /**
      * The process cannot access the file because it is being used by another process.
      */
-    NT_STATUS_SHARING_VIOLATION(CIFSExceptionMessages.NT_STATUS_SHARING_VIOLATION, CATEGORY_USER_INPUT),
+    NT_STATUS_SHARING_VIOLATION("The process cannot access the file because it is being used by another process.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_SHARING_VIOLATION),
     /**
      * Access is denied.
      */
-    NT_STATUS_DELETE_PENDING(CIFSExceptionMessages.NT_STATUS_DELETE_PENDING, CATEGORY_USER_INPUT),
+    NT_STATUS_DELETE_PENDING("Access is denied.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_DELETE_PENDING),
     /**
      * There are currently no logon servers available to service the logon request.
      */
-    NT_STATUS_NO_LOGON_SERVERS(CIFSExceptionMessages.NT_STATUS_NO_LOGON_SERVERS, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_LOGON_SERVERS("There are currently no logon servers available to service the logon request.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_NO_LOGON_SERVERS),
     /**
      * The specified user already exists.
      */
-    NT_STATUS_USER_EXISTS(CIFSExceptionMessages.NT_STATUS_USER_EXISTS, CATEGORY_USER_INPUT),
+    NT_STATUS_USER_EXISTS("The specified user already exists.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_USER_EXISTS),
     /**
      * The specified user does not exist.
      */
-    NT_STATUS_NO_SUCH_USER(CIFSExceptionMessages.NT_STATUS_NO_SUCH_USER, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_SUCH_USER("The specified user does not exist.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NO_SUCH_USER),
     /**
      * The specified network password is not correct.
      */
-    NT_STATUS_WRONG_PASSWORD(CIFSExceptionMessages.NT_STATUS_WRONG_PASSWORD, CATEGORY_USER_INPUT),
+    NT_STATUS_WRONG_PASSWORD("The specified network password is not correct.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_WRONG_PASSWORD),
     /**
      * Logon failure: unknown user name or bad password.
      */
-    NT_STATUS_LOGON_FAILURE(CIFSExceptionMessages.NT_STATUS_LOGON_FAILURE, CATEGORY_USER_INPUT),
+    NT_STATUS_LOGON_FAILURE("Logon failure: unknown user name or bad password.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_LOGON_FAILURE),
     /**
      * Logon failure: user account restriction.
      */
-    NT_STATUS_ACCOUNT_RESTRICTION(CIFSExceptionMessages.NT_STATUS_ACCOUNT_RESTRICTION, CATEGORY_USER_INPUT),
+    NT_STATUS_ACCOUNT_RESTRICTION("Logon failure: user account restriction.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_ACCOUNT_RESTRICTION),
     /**
      * Logon failure: account logon time restriction violation.
      */
-    NT_STATUS_INVALID_LOGON_HOURS(CIFSExceptionMessages.NT_STATUS_INVALID_LOGON_HOURS, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_LOGON_HOURS("Logon failure: account logon time restriction violation.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_INVALID_LOGON_HOURS),
     /**
      * Logon failure: user not allowed to log on to this computer.
      */
-    NT_STATUS_INVALID_WORKSTATION(CIFSExceptionMessages.NT_STATUS_INVALID_WORKSTATION, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_WORKSTATION("Logon failure: user not allowed to log on to this computer.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_INVALID_WORKSTATION),
     /**
      * Logon failure: the specified account password has expired.
      */
-    NT_STATUS_PASSWORD_EXPIRED(CIFSExceptionMessages.NT_STATUS_PASSWORD_EXPIRED, CATEGORY_USER_INPUT),
+    NT_STATUS_PASSWORD_EXPIRED("Logon failure: the specified account password has expired.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_PASSWORD_EXPIRED),
     /**
      * Logon failure: account currently disabled.
      */
-    NT_STATUS_ACCOUNT_DISABLED(CIFSExceptionMessages.NT_STATUS_ACCOUNT_DISABLED, CATEGORY_USER_INPUT),
+    NT_STATUS_ACCOUNT_DISABLED("Logon failure: account currently disabled.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_ACCOUNT_DISABLED),
     /**
      * No mapping between account names and security IDs was done.
      */
-    NT_STATUS_NONE_MAPPED(CIFSExceptionMessages.NT_STATUS_NONE_MAPPED, CATEGORY_USER_INPUT),
+    NT_STATUS_NONE_MAPPED("No mapping between account names and security IDs was done.", CATEGORY_ERROR, null),
     /**
      * The security ID structure is invalid.
      */
-    NT_STATUS_INVALID_SID(CIFSExceptionMessages.NT_STATUS_INVALID_SID, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_SID("The security ID structure is invalid.", CATEGORY_ERROR, null),
     /**
      * All pipe instances are busy.
      */
-    NT_STATUS_INSTANCE_NOT_AVAILABLE(CIFSExceptionMessages.NT_STATUS_INSTANCE_NOT_AVAILABLE, CATEGORY_USER_INPUT),
+    NT_STATUS_INSTANCE_NOT_AVAILABLE("All pipe instances are busy.", CATEGORY_ERROR, null),
     /**
      * All pipe instances are busy.
      */
-    NT_STATUS_PIPE_NOT_AVAILABLE(CIFSExceptionMessages.NT_STATUS_PIPE_NOT_AVAILABLE, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_NOT_AVAILABLE("All pipe instances are busy.", CATEGORY_ERROR, null),
     /**
      * The pipe state is invalid.
      */
-    NT_STATUS_INVALID_PIPE_STATE(CIFSExceptionMessages.NT_STATUS_INVALID_PIPE_STATE, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_PIPE_STATE("The pipe state is invalid.", CATEGORY_ERROR, null),
     /**
      * All pipe instances are busy.
      */
-    NT_STATUS_PIPE_BUSY(CIFSExceptionMessages.NT_STATUS_PIPE_BUSY, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_BUSY(CIFSExceptionMessages.NT_STATUS_PIPE_BUSY, CATEGORY_ERROR, null),
     /**
      * No process is on the other end of the pipe.
      */
-    NT_STATUS_PIPE_DISCONNECTED(CIFSExceptionMessages.NT_STATUS_PIPE_DISCONNECTED, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_DISCONNECTED("No process is on the other end of the pipe.", CATEGORY_ERROR, null),
     /**
      * The pipe is being closed.
      */
-    NT_STATUS_PIPE_CLOSING(CIFSExceptionMessages.NT_STATUS_PIPE_CLOSING, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_CLOSING("The pipe is being closed.", CATEGORY_ERROR, null),
     /**
      * Waiting for a process to open the other end of the pipe.
      */
-    NT_STATUS_PIPE_LISTENING(CIFSExceptionMessages.NT_STATUS_PIPE_LISTENING, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_LISTENING("Waiting for a process to open the other end of the pipe.", CATEGORY_TRY_AGAIN,
+        CIFSExceptionMessages.NT_STATUS_PIPE_LISTENING),
     /**
      * Access is denied.
      */
-    NT_STATUS_FILE_IS_A_DIRECTORY(CIFSExceptionMessages.NT_STATUS_FILE_IS_A_DIRECTORY, CATEGORY_USER_INPUT),
+    NT_STATUS_FILE_IS_A_DIRECTORY("Access is denied.", CATEGORY_PERMISSION_DENIED, CIFSExceptionMessages.NT_STATUS_FILE_IS_A_DIRECTORY),
     /**
      * A duplicate name exists on the network.
      */
-    NT_STATUS_DUPLICATE_NAME(CIFSExceptionMessages.NT_STATUS_DUPLICATE_NAME, CATEGORY_USER_INPUT),
+    NT_STATUS_DUPLICATE_NAME("A duplicate name exists on the network.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_DUPLICATE_NAME),
     /**
      * The specified network name is no longer available.
      */
-    NT_STATUS_NETWORK_NAME_DELETED(CIFSExceptionMessages.NT_STATUS_NETWORK_NAME_DELETED, CATEGORY_USER_INPUT),
+    NT_STATUS_NETWORK_NAME_DELETED("The specified network name is no longer available.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_NETWORK_NAME_DELETED),
     /**
      * Network access is denied.
      */
-    NT_STATUS_NETWORK_ACCESS_DENIED(CIFSExceptionMessages.NT_STATUS_NETWORK_ACCESS_DENIED, CATEGORY_USER_INPUT),
+    NT_STATUS_NETWORK_ACCESS_DENIED("Network access is denied.", CATEGORY_PERMISSION_DENIED,
+        CIFSExceptionMessages.NT_STATUS_NETWORK_ACCESS_DENIED),
     /**
      * The network name cannot be found.
      */
-    NT_STATUS_BAD_NETWORK_NAME(CIFSExceptionMessages.NT_STATUS_BAD_NETWORK_NAME, CATEGORY_USER_INPUT),
+    NT_STATUS_BAD_NETWORK_NAME("The network name cannot be found.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_BAD_NETWORK_NAME),
     /**
      * No more connections can be made to this remote computer at this time because there are already as many connections as the computer
      * can accept.
      */
-    NT_STATUS_REQUEST_NOT_ACCEPTED(CIFSExceptionMessages.NT_STATUS_REQUEST_NOT_ACCEPTED, CATEGORY_USER_INPUT),
+    NT_STATUS_REQUEST_NOT_ACCEPTED("No more connections can be made to this remote computer at this time because there are already as "
+        + "many connections as the computer can accept.", CATEGORY_TRY_AGAIN, CIFSExceptionMessages.NT_STATUS_REQUEST_NOT_ACCEPTED),
     /**
      * Indicates a Windows NT Server could not be contacted or that objects within the domain are protected such that necessary information
      * could not be retrieved.
      */
-    NT_STATUS_CANT_ACCESS_DOMAIN_INFO(CIFSExceptionMessages.NT_STATUS_CANT_ACCESS_DOMAIN_INFO, CATEGORY_USER_INPUT),
+    NT_STATUS_CANT_ACCESS_DOMAIN_INFO("Indicates a Windows NT Server could not be contacted or that objects within the domain are "
+        + "protected such that necessary information could not be retrieved.", CATEGORY_ERROR, null),
     /**
      * The specified domain did not exist.
      */
-    NT_STATUS_NO_SUCH_DOMAIN(CIFSExceptionMessages.NT_STATUS_NO_SUCH_DOMAIN, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_SUCH_DOMAIN("The specified domain did not exist.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NO_SUCH_DOMAIN),
     /**
      * The directory name is invalid.
      */
-    NT_STATUS_NOT_A_DIRECTORY(CIFSExceptionMessages.NT_STATUS_NOT_A_DIRECTORY, CATEGORY_USER_INPUT),
+    NT_STATUS_NOT_A_DIRECTORY("The directory name is invalid.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NOT_A_DIRECTORY),
     /**
      * Access is denied.
      */
-    NT_STATUS_CANNOT_DELETE(CIFSExceptionMessages.NT_STATUS_CANNOT_DELETE, CATEGORY_USER_INPUT),
+    NT_STATUS_CANNOT_DELETE("Access is denied.", CATEGORY_PERMISSION_DENIED, CIFSExceptionMessages.NT_STATUS_CANNOT_DELETE),
     /**
      * The format of the specified computer name is invalid.
      */
-    NT_STATUS_INVALID_COMPUTER_NAME(CIFSExceptionMessages.NT_STATUS_INVALID_COMPUTER_NAME, CATEGORY_USER_INPUT),
+    NT_STATUS_INVALID_COMPUTER_NAME("The format of the specified computer name is invalid.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_INVALID_COMPUTER_NAME),
     /**
      * The pipe has been ended.
      */
-    NT_STATUS_PIPE_BROKEN(CIFSExceptionMessages.NT_STATUS_PIPE_BROKEN, CATEGORY_USER_INPUT),
+    NT_STATUS_PIPE_BROKEN("The pipe has been ended.", CATEGORY_ERROR, null),
     /**
      * The specified local group does not exist.
      */
-    NT_STATUS_NO_SUCH_ALIAS(CIFSExceptionMessages.NT_STATUS_NO_SUCH_ALIAS, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_SUCH_ALIAS("The specified local group does not exist.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NO_SUCH_ALIAS),
     /**
      * Logon failure: the user has not been granted the requested logon type at this computer.
      */
-    NT_STATUS_LOGON_TYPE_NOT_GRANTED(CIFSExceptionMessages.NT_STATUS_LOGON_TYPE_NOT_GRANTED, CATEGORY_USER_INPUT),
+    NT_STATUS_LOGON_TYPE_NOT_GRANTED("Logon failure: the user has not been granted the requested logon type at this computer.",
+        CATEGORY_PERMISSION_DENIED, CIFSExceptionMessages.NT_STATUS_LOGON_TYPE_NOT_GRANTED),
     /**
      * The SAM database on the Windows NT Server does not have a computer account for this workstation trust relationship.
      */
-    NT_STATUS_NO_TRUST_SAM_ACCOUNT(CIFSExceptionMessages.NT_STATUS_NO_TRUST_SAM_ACCOUNT, CATEGORY_USER_INPUT),
+    NT_STATUS_NO_TRUST_SAM_ACCOUNT("The SAM database on the Windows NT Server does not have a computer account for this workstation "
+        + "trust relationship.", CATEGORY_ERROR, null),
     /**
      * The trust relationship between the primary domain and the trusted domain failed.
      */
-    NT_STATUS_TRUSTED_DOMAIN_FAILURE(CIFSExceptionMessages.NT_STATUS_TRUSTED_DOMAIN_FAILURE, CATEGORY_USER_INPUT),
+    NT_STATUS_TRUSTED_DOMAIN_FAILURE("The trust relationship between the primary domain and the trusted domain failed.", CATEGORY_ERROR
+        , null),
     /**
      * The account used is a Computer Account. Use your global user account or local user account to access this server.
      */
-    NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT(CIFSExceptionMessages.NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT, CATEGORY_USER_INPUT),
+    NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT("The account used is a Computer Account. Use your global user account or local user"
+        + " account to access this server.", CATEGORY_USER_INPUT, CIFSExceptionMessages.NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT),
     /**
      * The user must change his password before he logs on the first time.
      */
-    NT_STATUS_PASSWORD_MUST_CHANGE(CIFSExceptionMessages.NT_STATUS_PASSWORD_MUST_CHANGE, CATEGORY_USER_INPUT),
+    NT_STATUS_PASSWORD_MUST_CHANGE("The user must change his password before he logs on the first time.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_PASSWORD_MUST_CHANGE),
     /**
      * Please use a hostname instead of an IP address in the SMB URL.
      */
-    NT_STATUS_NOT_FOUND(CIFSExceptionMessages.NT_STATUS_NOT_FOUND, CATEGORY_USER_INPUT),
+    NT_STATUS_NOT_FOUND("Please use a hostname instead of an IP address in the SMB URL.", CATEGORY_USER_INPUT,
+        CIFSExceptionMessages.NT_STATUS_NOT_FOUND),
     /**
      * Your account has been blocked. This can have various reasons like having mistyped the password several times.<br>
      * Please contact your system administrator or hoster in case you can no longer log in.
      */
-    NT_STATUS_ACCOUNT_LOCKED_OUT(CIFSExceptionMessages.NT_STATUS_ACCOUNT_LOCKED_OUT, CATEGORY_USER_INPUT),
+    NT_STATUS_ACCOUNT_LOCKED_OUT("Your account has been blocked. This can have various reasons like having mistyped the password"
+        + " several times.\nPlease contact your system administrator or hoster in case you can no longer log in.",
+        CATEGORY_PERMISSION_DENIED, CIFSExceptionMessages.NT_STATUS_ACCOUNT_LOCKED_OUT),
     /**
      * The remote system is not reachable by the transport.
      */
-    NT_STATUS_PATH_NOT_COVERED(CIFSExceptionMessages.NT_STATUS_PATH_NOT_COVERED, CATEGORY_USER_INPUT),
+    NT_STATUS_PATH_NOT_COVERED("The remote system is not reachable by the transport.", CATEGORY_SERVICE_DOWN, null),
     /**
      * I/O request could not be handled.
      */
-    NT_STATUS_IO_REPARSE_TAG_NOT_HANDLED(CIFSExceptionMessages.NT_STATUS_IO_REPARSE_TAG_NOT_HANDLED, CATEGORY_USER_INPUT),
+    NT_STATUS_IO_REPARSE_TAG_NOT_HANDLED("I/O request could not be handled.", CATEGORY_ERROR, null),
 
     ;
 
@@ -452,11 +487,13 @@ public enum CIFSExceptionCodes implements OXExceptionCode {
     private final Category category;
     private final int detailNumber;
     private final String message;
+    private String displayMessage;
 
-    private CIFSExceptionCodes(final String message, final Category category) {
+    private CIFSExceptionCodes(final String message, final Category category, String displayMessage) {
         this.message = message;
         detailNumber = ordinal() + 1;
         this.category = category;
+        this.displayMessage = displayMessage != null ? displayMessage : OXExceptionStrings.MESSAGE;
     }
 
     private CIFSExceptionCodes(final String message, final Category category, final int detailNumber) {
@@ -474,6 +511,11 @@ public enum CIFSExceptionCodes implements OXExceptionCode {
     public String getMessage() {
         return message;
     }
+    
+    @Override
+    public String getDisplayMessage() {
+        return displayMessage;
+    }
 
     @Override
     public int getNumber() {
